package Security::freebsd;

use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT = qw( &init() &check() );

our (%perms);

sub init {
	$main::showmount = '/usr/bin/showmount';
	$main::exportsfile = '/etc/exports';
	$main::securefile='/etc/ttys';
	$main::mtree = '/usr/sbin/mtree';
	$main::sudofile = '/usr/local/etc/sudoers';
	# Find the directory where the loadable kernel modules are
	my $sysctl = '/usr/sbin/sysctl' if ( -f '/usr/sbin/sysctl' );
	$sysctl = '/sbin/sysctl' if ( -f '/sbin/sysctl' );
	if ( -f $sysctl ) {
		chop ($main::kerneldir=`$sysctl -n kern.module_path 2> /dev/null`);
		$main::kerneldir =~ s,^/;,^,o;
		$main::kerneldir =~ s/;/|^/go;
	}
}

sub check {
	&Security::rootlogin::check_ttys() 
		if ( defined &Security::rootlogin::check_ttys );

	&check_osfiles();
}

sub check_osfiles {
	my ($dir, $dir2, $file);
	local *FP;

	print ( STDERR "In Security::freebsd::check_osfiles()\n") if ( $main::opts{'d'} );

	if ( opendir (DIR, '/etc/mtree')) {  
		while ($file = readdir (DIR)) {
			$dir = '';
			$dir = '/'  if ( $file =~ /BSD.root.dist/ );
			$dir = '/var'  if ( $file =~ /BSD.var.dist/ );
			$dir = '/usr'  if ( $file =~ /BSD.usr.dist/ );
			$dir = '/usr/include'  if ( $file =~ /BSD.include.dist/ );
			$dir = '/usr/X11R6'  if ( $file =~ /BSD.x11.dist/ );
			$dir = '/usr/local'  if ( $file =~ /BSD.local.dist/ );
			if ( -d $dir && -f "/etc/mtree/$file" ) {
				print ( STDERR "\tRunning cmd $main::mtree -f /etc/mtree/$file -x -p $dir -e\n") if ( $main::opts{'d'} );
				open (FP, "$main::mtree -f /etc/mtree/$file -x -p $dir -e|") || die "Could not run $main::mtree\n";
				while (<FP>) {
					chop;
					if ( length $_ > 1 ) {
						if ( /missing$/o ) {
							$dir2 = $_;
							$dir2 =~ s, missing$,,o;
							$dir2 =~ s,^\.,,o;
							if ( $dir ne '/' ) {
								$dir2 = (( $dir2 =~ m|^/|)
									? "$dir$dir2"
									: "$dir/$dir2");
							}
							$perms{$dir2} = "file not found";
						} elsif ( /.* changed$/o ) {
							$dir2 = $_;
							$dir2 =~ s/ changed$//go;
							$dir2 =~ s,^\.,,o;
							if ( $dir ne '/' ) {
								$dir2 = (( $dir2 =~ m|^/|)
									? "$dir$dir2"
									: "$dir/$dir2");
							}
							$perms{$dir2} = 1;
						} elsif ( exists $perms{$dir2} ) {
							$perms{$dir2}=($perms{$dir2} eq '1')
								? $_: $perms{$dir2} . $_;
							$perms{$dir2} =~ s/  / /go;
						}
					}
				}
				close FP;
			}
		}
		closedir DIR;
	}
}

sub print {
	print ( STDERR  "In Security::freebsd::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking mtree command output:\n", %perms);
}

1;
